#include <Wire.h>
#include <Adafruit_INA219.h>
#include <ESP32Servo.h>
#include <VL53L0X.h>    
#define I2C_SDA 14
#define I2C_SCL 15

Adafruit_INA219 ina219;


Servo myservo;  // create servo object to control a servo
int servoPin = 2;

VL53L0X sensor;

//Servo position open and closed
int servoPosOpen = 140;
int servoPosClosed = 5;

//pin number
const int StartProgPin = 16;
const int PositionOK = 12;
const int GripperClosedPin = 0;
const int OverLoad = 13; 
//state
int StartProg = 0;
int GripperClosed = 0;

void setup()
{
Serial.begin(115200);
  Wire.begin (I2C_SDA, I2C_SCL);
  while (!Serial) {
      // will pause Zero, Leonardo, etc until serial console opens
      delay(1);
  }
//declaring pinmodes 
  pinMode(PositionOK, OUTPUT);
  pinMode(OverLoad, OUTPUT);
  pinMode(GripperClosed, INPUT);
  pinMode(StartProg, INPUT); 
  digitalWrite(OverLoad, HIGH);
  digitalWrite(PositionOK, LOW); 
  
  uint32_t currentFrequency;
  
  // Initialize the INA219.
  // By default the initialization will use the largest range (32V, 2A).  However
  // you can call a setCalibration function to change this range (see comments).
  if (! ina219.begin()) {
    Serial.println("Failed to find INA219 chip");
    while (1) { delay(10); }
  }
  // To use a slightly lower 32V, 1A range (higher precision on amps):
  //ina219.setCalibration_32V_1A();
  // Or to use a lower 16V, 400mA range (higher precision on volts and amps):
  //ina219.setCalibration_16V_400mA();

  Serial.println("Measuring voltage and current with INA219 ..."); 

  // Allow allocation of all timers
  ESP32PWM::allocateTimer(0);
  ESP32PWM::allocateTimer(1);
  ESP32PWM::allocateTimer(2);
  ESP32PWM::allocateTimer(3);
  myservo.setPeriodHertz(50);    // standard 50 hz servo
  myservo.attach(servoPin, 1000, 2000); // attaches the servo on pin 18 to the servo object
  // using default min/max of 1000us and 2000us
  // different servos may require different min/max settings
  // for an accurate 0 to 180 sweep

 sensor.setTimeout(500);
  if (!sensor.init())
  {
    Serial.println("Failed to detect and initialize sensor!");
    while (1) {}
  }

  // Start continuous back-to-back mode (take readings as
  // fast as possible).  To use continuous timed mode
  // instead, provide a desired inter-measurement period in
  // ms (e.g. sensor.startContinuous(100)).
  sensor.startContinuous();

}

void loop() {
  
  float shuntvoltage = 0;
  float busvoltage = 0;
  float current_mA = 0;
  float loadvoltage = 0;
  float power_mW = 0;

  shuntvoltage = ina219.getShuntVoltage_mV();
  busvoltage = ina219.getBusVoltage_V();
  current_mA = ina219.getCurrent_mA();
  power_mW = ina219.getPower_mW();
  loadvoltage = busvoltage + (shuntvoltage / 1000);

  /* //If you want to print out what the INA219 is measuring 
  Serial.print("Bus Voltage:   "); Serial.print(busvoltage); Serial.println(" V");
  Serial.print("Shunt Voltage: "); Serial.print(shuntvoltage); Serial.println(" mV");
  Serial.print("Load Voltage:  "); Serial.print(loadvoltage); Serial.println(" V");
  Serial.print("Current:       "); Serial.print(current_mA); Serial.println(" mA");
  Serial.print("Power:         "); Serial.print(power_mW); Serial.println(" mW");
  Serial.println("");
  */
  delay(50);
  //kode fra VL53LOX_sensor
  //if you want to print what the VL53LOX sensor is measuring   
  //Serial.print(sensor.readRangeContinuousMillimeters());
  if (sensor.timeoutOccurred()) { Serial.print(" TIMEOUT"); }
  
  //reading the measured distance
  int measuredDistance;
  measuredDistance = sensor.readRangeContinuousMillimeters();
  delay(50);
  
  //reading the digital pins 
  StartProg = digitalRead(StartProgPin);
  GripperClosed = digitalRead(GripperClosedPin);
  
  //Venter på start signal fra UR 
  if (StartProg == true){
     //opening the gripper tool 
     myservo.write(servoPosClosed); 
  }
  else {
     //closeing the gripper tool 
     myservo.write(servoPosOpen); 
     }
  //waiting for the right position for the gripper   
  if (measuredDistance < 90){
    //giving signal to the UR that the position is right.
    digitalWrite(PositionOK, HIGH);
    //myservo.write(servoPosOpen);
    //closning the gripper and harvest the plant
    myservo.write(servoPosClosed);
    }
  else{ 
    digitalWrite(PositionOK, LOW); 
    }    
    //Signal from UR that the gripper tool is closed 
    if (GripperClosed == true){ 
     digitalWrite(PositionOK, LOW);
    }  
    //gives a signal to the UR if there is over load on the servomotor(MG995 = 1200.00mA and the MG995 = 2500.00mA)
    if (current_mA > 2500.00){
    digitalWrite(OverLoad, HIGH);
    }
    else{
    digitalWrite(OverLoad, LOW);
    }  
    delay(50);

}
